@@ -33,6 +33,10 @@ void DEploidIO::wrapUp(){
     // Get End time before writing the log
     this->getTime(false);
 
+    if (this->useBestPractice()){
+        this->writeChooseKProportion();
+    }
+
     this->writeLog (&std::cout);
 
     ofstreamExportTmp.open( strExportLog.c_str(), ios::out | ios::app | ios::binary );
@@ -100,22 +104,26 @@ void DEploidIO::writeLog ( ostream * writeTo ){
         if (this->useIBD()){
             (*writeTo) << setw(19) << "  IBD Method used: YES" << "\n";
         }
-        (*writeTo) << setw(19) << " Update Prop: "   << (this->doUpdateProp()  ? "YES":"NO") << "\n";
-        (*writeTo) << setw(19) << " Update Single: " << (this->doUpdateSingle()? "YES":"NO") << "\n";
-        (*writeTo) << setw(19) << " Update Pair: "   << (this->doUpdatePair()  ? "YES":"NO") << "\n";
+        (*writeTo) << setw(19) << " Update Prop: "
+                   << (this->doUpdateProp()  ? "YES":"NO") << "\n";
+        (*writeTo) << setw(19) << " Update Single: "
+                   << (this->doUpdateSingle()? "YES":"NO") << "\n";
+        (*writeTo) << setw(19) << " Update Pair: "
+                   << (this->doUpdatePair()  ? "YES":"NO") << "\n";
         (*writeTo) << "\n";
     }
     (*writeTo) << "Other parameters:"<< "\n";
     if ( forbidCopyFromSame_ ){ (*writeTo) << " Update pair haplotypes move forbid copying from the same strain!!! \n"; }
     (*writeTo) << setw(20) << " Miss copy prob: "   << this->missCopyProb_ << "\n";
     (*writeTo) << setw(20) << " Avrg Cent Morgan: " << this->averageCentimorganDistance_ << "\n";
-    (*writeTo) << setw(20) << " G: "               << this->parameterG() << "\n";
+    (*writeTo) << setw(20) << " G: "                << this->parameterG() << "\n";
     if (this->useIBD()){
-    (*writeTo) << setw(20) << " IBD sigma: "               << this->ibdSigma() << "\n";
+    (*writeTo) << setw(20) << " IBD sigma: "        << this->ibdSigma() << "\n";
     } else {
-    (*writeTo) << setw(20) << " sigma: "               << this->parameterSigma() << "\n";
+    (*writeTo) << setw(20) << " sigma: "            << this->parameterSigma() << "\n";
     }
     (*writeTo) << setw(20) << " ScalingFactor: "    << this->scalingFactor() << "\n";
+    (*writeTo) << setw(20) << " VQSLOD:        "    << this->vqslod() << "\n";
     if ( this->initialPropWasGiven() ){
         (*writeTo) << setw(20) << " Initial prob: " ;
         for ( size_t i = 0; i < this->initialProp.size(); i++ ){
@@ -139,6 +147,13 @@ void DEploidIO::writeLog ( ostream * writeTo ){
     (*writeTo) << setw(14) << "Start at: "  << startingTime_  ;
     (*writeTo) << setw(14) << "End at: "    << endTime_  ;
     (*writeTo) << "\n";
+    if (this->useBestPractice()){
+        (*writeTo) << "Confidence: " << endl;
+        (*writeTo) << setw(14) << "k: "
+                   << this->chooseK.confidence() * 100 << "%" << endl;
+        (*writeTo) << "\n";
+    }
+
     if ( this->doComputeLLK() ){
         (*writeTo) << "Input likelihood: " << llkFromInitialHap_;
         (*writeTo) << "\n";
@@ -157,17 +172,28 @@ void DEploidIO::writeLog ( ostream * writeTo ){
                 }
             }
         } else {
-            if (this->useLasso() == false) {
-                (*writeTo) << setw(14) << "Likelihood: "  << strExportLLK  << "\n";
-                (*writeTo) << setw(14) << "Proportions: " << strExportProp << "\n";
-            }
-            (*writeTo) << setw(14) << "Haplotypes: "  << strExportHap  << "\n";
+            //if (this->useLasso() == false) {
+                //(*writeTo) << setw(14) << "Likelihood: "  << strExportLLK  << "\n";
+                //(*writeTo) << setw(14) << "Proportions: " << strExportProp << "\n";
+            //}
+            //(*writeTo) << setw(14) << "Haplotypes: "  << strExportHap  << "\n";
             if ( doExportVcf() ) { (*writeTo) << setw(14) << "Vcf: "  << strExportVcf  << "\n"; }
             if (this->useIBD()){
                 (*writeTo) << " IBD method output saved to:\n";
-                (*writeTo) << setw(14) << "Likelihood: "  << strIbdExportLLK  << "\n";
-                (*writeTo) << setw(14) << "Proportions: " << strIbdExportProp << "\n";
-                (*writeTo) << setw(14) << "Haplotypes: "  << strIbdExportHap  << "\n";
+                (*writeTo) << setw(14) << "Likelihood: "  << this->prefix_ << ".*.llk\n";
+                (*writeTo) << setw(14) << "Proportions: " << this->prefix_ << ".*.prop\n";
+                (*writeTo) << setw(14) << "Haplotypes: "  << this->prefix_ << ".*.hap\n";
+            } else if(this->useBestPractice()){
+              (*writeTo) << setw(14) << "ChooseK Haps: "  << this->prefix_ << ".chooseK.hap\n";
+              (*writeTo) << setw(14) << "IBD Haps: "  << this->prefix_ << ".ibd.hap\n";
+              (*writeTo) << setw(14) << "Final Haps: "  << this->prefix_ << ".final.hap\n";
+              (*writeTo) << setw(14) << "IBD llks: "  << this->prefix_ << ".ibd.llk\n";
+              (*writeTo) << setw(14) << "Proportions: " << this->prefix_ << ".chooseK.prop\n";
+              (*writeTo) << setw(14) << "IBD props: " << this->prefix_ << ".ibd.prop\n";
+            } else {
+              (*writeTo) << setw(14) << "Likelihood: "  << this->prefix_ << ".llk\n";
+              (*writeTo) << setw(14) << "Proportions: " << this->prefix_ << ".prop\n";
+              (*writeTo) << setw(14) << "Haplotypes: "  << this->prefix_ << ".hap\n";
             }
             if (this->ibdProbsIntegrated.size()>1){
                 (*writeTo) << setw(14) << "IBD probs: "  << strIbdExportProbs  << "\n\n";
@@ -228,28 +254,28 @@ void DEploidIO::writeEventCount(){
     for ( size_t chromI = 0; chromI < chrom_.size(); chromI++ ){
         for ( size_t posI = 0; posI < position_[chromI].size(); posI++){
             ofstreamExportTmp << chrom_[chromI] << "\t"
-                              << (int)position_[chromI][posI] << "\t"
+                          << (int)position_[chromI][posI] << "\t"
 
-                              << this->IBDpathChangeAt[siteIndex] << "\t"
-                              << this->finalIBDpathChangeAt[siteIndex] << "\t"
+                          << this->IBDpathChangeAt[siteIndex] << "\t"
+                          << this->finalIBDpathChangeAt[siteIndex] << "\t"
 
-                              << this->siteOfTwoSwitchOne[siteIndex] << "\t"
-                              << this->finalSiteOfTwoSwitchOne[siteIndex] << "\t"
+                          << this->siteOfTwoSwitchOne[siteIndex] << "\t"
+                          << this->finalSiteOfTwoSwitchOne[siteIndex] << "\t"
 
-                              << this->siteOfTwoMissCopyOne[siteIndex] << "\t"
-                              << this->finalSiteOfTwoMissCopyOne[siteIndex] << "\t"
+                          << this->siteOfTwoMissCopyOne[siteIndex] << "\t"
+                          << this->finalSiteOfTwoMissCopyOne[siteIndex] << "\t"
 
-                              << this->siteOfTwoSwitchTwo[siteIndex] << "\t"
-                              << this->finalSiteOfTwoSwitchTwo[siteIndex] << "\t"
+                          << this->siteOfTwoSwitchTwo[siteIndex] << "\t"
+                          << this->finalSiteOfTwoSwitchTwo[siteIndex] << "\t"
 
-                              << this->siteOfTwoMissCopyTwo[siteIndex] << "\t"
-                              << this->finalSiteOfTwoMissCopyTwo[siteIndex] << "\t"
+                          << this->siteOfTwoMissCopyTwo[siteIndex] << "\t"
+                          << this->finalSiteOfTwoMissCopyTwo[siteIndex] << "\t"
 
-                              << this->siteOfOneSwitchOne[siteIndex] << "\t"
-                              << this->finalSiteOfOneSwitchOne[siteIndex] << "\t"
+                          << this->siteOfOneSwitchOne[siteIndex] << "\t"
+                          << this->finalSiteOfOneSwitchOne[siteIndex] << "\t"
 
-                              << this->siteOfOneMissCopyOne[siteIndex] << "\t"
-                              << this->finalSiteOfOneMissCopyOne[siteIndex] << endl;
+                          << this->siteOfOneMissCopyOne[siteIndex] << "\t"
+                          << this->finalSiteOfOneMissCopyOne[siteIndex] << endl;
             siteIndex++;
         }
     }
@@ -294,6 +320,36 @@ void DEploidIO::writeIBDpostProb(vector < vector <double> > & reshapedProbs, vec
 }
 
 
+void DEploidIO::writeIBDviterbi(vector <size_t> & viterbiState){
+    ostream * writeTo;
+    #ifdef UNITTEST
+        writeTo = &std::cout;
+    #endif
+
+    #ifndef UNITTEST
+        ofstreamExportTmp.open( strIbdExportViterbi.c_str(), ios::out | ios::app | ios::binary );
+        writeTo = &ofstreamExportTmp;
+    #endif
+
+    (*writeTo) << "CHROM" << "\t" << "POS" << "\t" << "viterbi" << "\n" ;
+
+    size_t siteIndex = 0;
+    for ( size_t chromIndex = 0; chromIndex < position_.size(); chromIndex++){
+        for ( size_t posI = 0; posI < position_[chromIndex].size(); posI++){
+            (*writeTo) << chrom_[chromIndex] << "\t"
+                       << (int)position_[chromIndex][posI] << "\t"
+                       << (int)viterbiState[siteIndex] << "\n";
+            siteIndex++;
+        }
+    }
+    assert(siteIndex == nLoci());
+
+    #ifndef UNITTEST
+        ofstreamExportTmp.close();
+    #endif
+}
+
+
 void DEploidIO::paintIBD(){
     vector <double> goodProp;
     vector <size_t> goodStrainIdx;
@@ -324,10 +380,11 @@ void DEploidIO::paintIBD(){
     tmpDEploidIO.nLoci_= this->nLoci();
     tmpDEploidIO.position_ = this->position_;
     tmpDEploidIO.chrom_ = this->chrom_;
-    //tmpDEploidIO.useConstRecomb_ = true;
-    //tmpDEploidIO.constRecombProb_ = 0.000001;
+    tmpDEploidIO.setParameterG(this->parameterG());
+    // tmpDEploidIO.useConstRecomb_ = true;
+    // tmpDEploidIO.constRecombProb_ = 0.000001;
 
-    //tmpDEploidIO.writeLog (&std::cout);
+    // tmpDEploidIO.writeLog (&std::cout);
 
     MersenneTwister tmpRg(this->randomSeed());
     IBDpath tmpIBDpath;
@@ -336,6 +393,77 @@ void DEploidIO::paintIBD(){
     this->ibdLLK_ = tmpIBDpath.bestPath(goodProp);
     this->ibdProbsHeader = tmpIBDpath.getIBDprobsHeader();
     this->getIBDprobsIntegrated(tmpIBDpath.fwdbwd);
+
+
+    for ( size_t stateI = 0; stateI < this->ibdProbsHeader.size(); stateI++ ){
+        cout << setw(14) << this->ibdProbsHeader[stateI] << ": "
+             << this->ibdProbsIntegrated[stateI] << "\n";
+    }
+
     this->writeIBDpostProb(tmpIBDpath.fwdbwd, this->ibdProbsHeader);
 }
 
+
+void DEploidIO::paintIBDviterbi(){
+    vector <double> goodProp;
+    vector <size_t> goodStrainIdx;
+
+    if ( this->doIbdViterbiPainting() ){
+        this->finalProp = this->initialProp;
+    }
+
+    for ( size_t i = 0; i < this->finalProp.size(); i++){
+        if (this->finalProp[i] > 0.01){
+            goodProp.push_back(this->finalProp[i]);
+            goodStrainIdx.push_back(i);
+        }
+    }
+
+    if (goodProp.size() == 1){
+        return;
+    }
+
+    DEploidIO tmpDEploidIO; // (*this);
+    tmpDEploidIO.setKstrain(goodProp.size());
+    tmpDEploidIO.setInitialPropWasGiven(true);
+    tmpDEploidIO.initialProp = goodProp;
+    tmpDEploidIO.finalProp = goodProp;
+    tmpDEploidIO.refCount_ = this->refCount_;
+    tmpDEploidIO.altCount_ = this->altCount_;
+    tmpDEploidIO.plaf_ = this->plaf_;
+    tmpDEploidIO.nLoci_= this->nLoci();
+    tmpDEploidIO.position_ = this->position_;
+    tmpDEploidIO.chrom_ = this->chrom_;
+    tmpDEploidIO.setParameterG(this->parameterG());
+     //tmpDEploidIO.useConstRecomb_ = true;
+     //tmpDEploidIO.constRecombProb_ = 0.000001;
+
+    // tmpDEploidIO.writeLog (&std::cout);
+
+    MersenneTwister tmpRg(this->randomSeed());
+    IBDpath tmpIBDpath;
+    tmpIBDpath.init(tmpDEploidIO, &tmpRg);
+
+    //tmpIBDpath.buildPathProbabilityForPainting(goodProp);
+    //tmpIBDpath.findViterbiPath(goodProp);
+    this->ibdLLK_ = tmpIBDpath.findViterbiPath(goodProp);//''tmpIBDpath.bestPath(goodProp);
+    //this->ibdProbsHeader = tmpIBDpath.getIBDprobsHeader();
+    //this->getIBDprobsIntegrated(tmpIBDpath.fm);
+
+    this->writeIBDviterbi(tmpIBDpath.viterbiPath);
+}
+
+
+void DEploidIO::writeChooseKProportion() {
+    string fileName = this->prefix_ + ".chooseK.prop";
+    ofstreamExportTmp.open(fileName.c_str(), ios::out | ios::binary);
+    ostream * writeTo = &ofstreamExportTmp;
+    for (size_t i = 0; i < this->chooseK.proportions_.size(); i++) {
+        for (size_t ii = 0; ii < this->chooseK.proportions_[i].size(); ii++) {
+            (*writeTo)  << this->chooseK.proportions_[i][ii];
+            (*writeTo)  << (ii == (this->chooseK.proportions_[i].size()-1) ?
+                        "\n":"\t");
+        }
+    }
+    ofstreamExportTmp.close();
+}
